from functools import wraps
from .route_holder import RouteHolder

def jsonapi(reject_if_no_data=False):
  def decorator_wrapper(func):
    holder = RouteHolder()
    holder.add_route_partial(func, jsonapi=True)

    @wraps(func)
    def func_wrapper(req, *args, **kwArgs):
      if reject_if_no_data and not req.data:
        respondent.set_error("Body is not a valid JSONAPI object")
        respondent.set_status(400)
        return
        
      return func(*args, **kwArgs)
    return func_wrapper
  return decorator_wrapper



import sys
from functools import wraps
from .route_holder import RouteHolder

module_obj = sys.modules[__name__]
VERBS = ['get', 'post', 'put', 'delete', 'patch', 'options', 'head']

for verb in VERBS:
  def make_me_a_method(inverb):
    def http_method_wrapper(func):
      holder = RouteHolder()
      holder.add_route_partial(func, http_method=inverb.upper())

      @wraps(func)
      def func_wrapper(*args, **kwArgs):
        return func(*args, **kwArgs)
      return func_wrapper
    return http_method_wrapper
  
  func_instance = make_me_a_method(verb)
  func_instance.__name__ = verb
  func_instance.__module__ = module_obj
  setattr(module_obj, verb, func_instance)
  
__all__ = VERBS


from dpath.util import get
from functools import wraps
from ..route_holder import RouteHolder

def _no_blank(value):
  """must not be blank"""
  return value == ''

_VALIDATIONS_FAILS = {
  'no-blank': _no_blank
}

def _required_source_param(req_key, error_stub, param_path, *, field_name, validations):
  def decorator_wrapper(func):
    @wraps(func)
    def func_wrapper(req, *, respondent, **kwArgs):
      public_name = param_path.split('/')[-1]
      if field_name is not None:
        public_name = field_name
      try:
        value = str(get(getattr(req, req_key), param_path))
      except KeyError:
        respondent.set_error("Missing required {} {}".format(error_stub, public_name))
        respondent.set_status(400)
        return

      for validation_type in validations:
        if validation_type in _VALIDATIONS_FAILS:
          if _VALIDATIONS_FAILS[validation_type](value):
            respondent.set_error("{cap_error_stub} '{field_name}' {explanation}".format(
              cap_error_stub=error_stub.capitalize(),
              field_name=public_name,
              explanation=_VALIDATIONS_FAILS[validation_type].__doc__
            ))
            respondent.set_status(400)
            return

      return func(*args, **kwArgs)
    return func_wrapper
  return decorator_wrapper


def requires_body_param(param_path, field_name=None, validations=[]):
  return _required_source_param('body', 'field', param_path, field_name=field_name, validations=validations)

def requires_data_param(param_path, field_name=None, validations=[]):
  return _required_source_param('data', 'field', param_path, field_name=field_name, validations=validations)

__all__ = [
  'requires_data_param',
  'requires_body_param'
]